@@ -9,32 +9,38 @@
 //! with random hash functions.  So each subsequent request will have a different distribution
 //! of false positives.
 
-use crate::{
-    cluster_info::{Ping, CRDS_UNIQUE_PUBKEY_CAPACITY},
-    contact_info::ContactInfo,
-    crds::Crds,
-    crds_gossip::{get_stake, get_weight},
-    crds_gossip_error::CrdsGossipError,
-    crds_value::CrdsValue,
-    ping_pong::PingCache,
-};
-use itertools::Itertools;
-use lru::LruCache;
-use rand::distributions::{Distribution, WeightedIndex};
-use rand::Rng;
-use rayon::{prelude::*, ThreadPool};
-use solana_runtime::bloom::{AtomicBloom, Bloom};
-use solana_sdk::{
-    hash::{hash, Hash},
-    pubkey::Pubkey,
-    signature::{Keypair, Signer},
-};
-use std::{
-    collections::{HashMap, HashSet, VecDeque},
-    convert::TryInto,
-    net::SocketAddr,
-    sync::Mutex,
-    time::{Duration, Instant},
+use {
+    crate::{
+        cluster_info::{Ping, CRDS_UNIQUE_PUBKEY_CAPACITY},
+        contact_info::ContactInfo,
+        crds::Crds,
+        crds_gossip::{get_stake, get_weight},
+        crds_gossip_error::CrdsGossipError,
+        crds_value::CrdsValue,
+        ping_pong::PingCache,
+    },
+    lru::LruCache,
+    rand::{
+        distributions::{Distribution, WeightedIndex},
+        Rng,
+    },
+    rayon::{prelude::*, ThreadPool},
+    solana_runtime::bloom::{AtomicBloom, Bloom},
+    solana_sdk::{
+        hash::{hash, Hash},
+        pubkey::Pubkey,
+        signature::{Keypair, Signer},
+    },
+    std::{
+        collections::{HashMap, HashSet, VecDeque},
+        convert::TryInto,
+        net::SocketAddr,
+        sync::{
+            atomic::{AtomicI64, AtomicUsize, Ordering},
+            Mutex,
+        },
+        time::{Duration, Instant},
+    },
 };
 
 pub const CRDS_GOSSIP_PULL_CRDS_TIMEOUT_MS: u64 = 15000;
@@ -333,12 +339,13 @@ impl CrdsGossipPull {
     /// Create gossip responses to pull requests
     pub fn generate_pull_responses(
         &self,
+        thread_pool: &ThreadPool,
         crds: &Crds,
         requests: &[(CrdsValue, CrdsFilter)],
         output_size_limit: usize, // Limit number of crds values returned.
         now: u64,
     ) -> Vec<Vec<CrdsValue>> {
-        self.filter_crds_values(crds, requests, output_size_limit, now)
+        self.filter_crds_values(thread_pool, crds, requests, output_size_limit, now)
     }
 
     // Checks if responses should be inserted and
@@ -472,9 +479,10 @@ impl CrdsGossipPull {
     /// filter values that fail the bloom filter up to max_bytes
     fn filter_crds_values(
         &self,
+        thread_pool: &ThreadPool,
         crds: &Crds,
         filters: &[(CrdsValue, CrdsFilter)],
-        mut output_size_limit: usize, // Limit number of crds values returned.
+        output_size_limit: usize, // Limit number of crds values returned.
         now: u64,
     ) -> Vec<Vec<CrdsValue>> {
         let msg_timeout = CRDS_GOSSIP_PULL_CRDS_TIMEOUT_MS;
@@ -482,46 +490,53 @@ impl CrdsGossipPull {
         //skip filters from callers that are too old
         let caller_wallclock_window =
             now.saturating_sub(msg_timeout)..now.saturating_add(msg_timeout);
-        let mut dropped_requests = 0;
-        let mut total_skipped = 0;
-        let ret: Vec<_> = filters
-            .iter()
-            .map(|(caller, filter)| {
-                if output_size_limit == 0 {
-                    return None;
-                }
-                let caller_wallclock = caller.wallclock();
-                if !caller_wallclock_window.contains(&caller_wallclock) {
-                    dropped_requests += 1;
-                    return Some(vec![]);
-                }
-                let caller_wallclock = caller_wallclock.checked_add(jitter).unwrap_or(0);
-                let out: Vec<_> = crds
-                    .filter_bitmask(filter.mask, filter.mask_bits)
-                    .filter_map(|item| {
-                        debug_assert!(filter.test_mask(&item.value_hash));
-                        //skip values that are too new
-                        if item.value.wallclock() > caller_wallclock {
-                            total_skipped += 1;
-                            None
-                        } else if filter.filter_contains(&item.value_hash) {
-                            None
-                        } else {
-                            Some(item.value.clone())
-                        }
-                    })
-                    .take(output_size_limit)
-                    .collect();
-                output_size_limit -= out.len();
-                Some(out)
-            })
-            .while_some()
-            .collect();
+        let dropped_requests = AtomicUsize::default();
+        let total_skipped = AtomicUsize::default();
+        let output_size_limit = output_size_limit.try_into().unwrap_or(i64::MAX);
+        let output_size_limit = AtomicI64::new(output_size_limit);
+        let apply_filter = |caller: &CrdsValue, filter: &CrdsFilter| {
+            if output_size_limit.load(Ordering::Relaxed) <= 0 {
+                return Vec::default();
+            }
+            let caller_wallclock = caller.wallclock();
+            if !caller_wallclock_window.contains(&caller_wallclock) {
+                dropped_requests.fetch_add(1, Ordering::Relaxed);
+                return Vec::default();
+            }
+            let caller_wallclock = caller_wallclock.checked_add(jitter).unwrap_or(0);
+            let out: Vec<_> = crds
+                .filter_bitmask(filter.mask, filter.mask_bits)
+                .filter_map(|item| {
+                    debug_assert!(filter.test_mask(&item.value_hash));
+                    //skip values that are too new
+                    if item.value.wallclock() > caller_wallclock {
+                        total_skipped.fetch_add(1, Ordering::Relaxed);
+                        None
+                    } else if filter.filter_contains(&item.value_hash) {
+                        None
+                    } else {
+                        Some(item.value.clone())
+                    }
+                })
+                .take(output_size_limit.load(Ordering::Relaxed).max(0) as usize)
+                .collect();
+            output_size_limit.fetch_sub(out.len() as i64, Ordering::Relaxed);
+            out
+        };
+        let ret: Vec<_> = thread_pool.install(|| {
+            filters
+                .par_iter()
+                .map(|(caller, filter)| apply_filter(caller, filter))
+                .collect()
+        });
         inc_new_counter_info!(
             "gossip_filter_crds_values-dropped_requests",
-            dropped_requests + filters.len() - ret.len()
+            dropped_requests.into_inner()
+        );
+        inc_new_counter_info!(
+            "gossip_filter_crds_values-dropped_values",
+            total_skipped.into_inner()
         );
-        inc_new_counter_info!("gossip_filter_crds_values-dropped_values", total_skipped);
         ret
     }
 
@@ -1098,10 +1113,11 @@ pub(crate) mod tests {
         let (_, filters) = req.unwrap();
         let mut filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
         let rsp = dest.generate_pull_responses(
+            &thread_pool,
             &dest_crds,
             &filters,
-            /*output_size_limit=*/ usize::MAX,
-            0,
+            usize::MAX, // output_size_limit
+            0,          // now
         );
 
         assert_eq!(rsp[0].len(), 0);
@@ -1116,10 +1132,11 @@ pub(crate) mod tests {
 
         //should skip new value since caller is to old
         let rsp = dest.generate_pull_responses(
+            &thread_pool,
             &dest_crds,
             &filters,
-            /*output_size_limit=*/ usize::MAX,
-            CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS,
+            usize::MAX,                      // output_size_limit
+            CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS, // now
         );
         assert_eq!(rsp[0].len(), 0);
         assert_eq!(filters.len(), MIN_NUM_BLOOM_FILTERS);
@@ -1134,9 +1151,10 @@ pub(crate) mod tests {
                 .collect::<Vec<_>>()
         });
         let rsp = dest.generate_pull_responses(
+            &thread_pool,
             &dest_crds,
             &filters,
-            /*output_size_limit=*/ usize::MAX,
+            usize::MAX, // output_size_limit
             CRDS_GOSSIP_PULL_MSG_TIMEOUT_MS,
         );
         assert_eq!(rsp.len(), 2 * MIN_NUM_BLOOM_FILTERS);
@@ -1186,10 +1204,11 @@ pub(crate) mod tests {
         let (_, filters) = req.unwrap();
         let filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
         let rsp = dest.generate_pull_responses(
+            &thread_pool,
             &dest_crds,
             &filters,
-            /*output_size_limit=*/ usize::MAX,
-            0,
+            usize::MAX, // output_size_limit
+            0,          // now
         );
         dest.process_pull_requests(
             &mut dest_crds,
@@ -1258,10 +1277,11 @@ pub(crate) mod tests {
             let (_, filters) = req.unwrap();
             let filters: Vec<_> = filters.into_iter().map(|f| (caller.clone(), f)).collect();
             let rsp = dest.generate_pull_responses(
+                &thread_pool,
                 &dest_crds,
                 &filters,
-                /*output_size_limit=*/ usize::MAX,
-                0,
+                usize::MAX, // output_size_limit
+                0,          // now
             );
             dest.process_pull_requests(
                 &mut dest_crds,
